#include "StdAfx.h"

#include "VideoManager.h"

/**********************************************************************/
CVideoManager::CVideoManager(void)
{
	_colorImageData = NULL;
	_grayImageData  = NULL;
	_colorHeader    = NULL;
	_grayHeader     = NULL;

	_isVideoStreamOpen = false;

	_maxVChannelNumber  = 0;
	_videoChannelNumber = 0;

	_videoDeviceName = _T("No video device available");

	_colorImageSize.width        = 0; 
	_colorImageSize.height       = 0; 
	_colorImageSize.colorChannel = 0;

	_grayImageSize.width         = 0; 
	_grayImageSize.height        = 0;
	_grayImageSize.colorChannel  = 0;

	_lastColorImageSize.width        = 0; 
	_lastColorImageSize.height       = 0; 
	_lastColorImageSize.colorChannel = 0;

	for(int i = 0; i < MAX_DEVICE_NUM; i++)
		_videoDeviceList[i].Empty(); 

	PrepareVideoDeviceProperty();
}
CVideoManager::~CVideoManager(void)
{
	for(int i = 0; i < MAX_DEVICE_NUM; i++)
		_videoDeviceList[i].Empty(); 

	if(_videoProrertyPtr.GetInterfacePtr())
		_videoProrertyPtr.Release(); 

	if(_videoModulePtr.GetInterfacePtr())
		_videoModulePtr.Release();
}
/**********************************************************************/
void CVideoManager::PrepareVideoDeviceProperty()
{
	DeviceList deviceList;

	try
	{
		if(!_videoProrertyPtr.GetInterfacePtr()) 
			_videoProrertyPtr = IVideoDevicePropertyPtr(__uuidof(VideoDeviceProperty));
	}
	catch(_com_error err)
		{
			AfxMessageBox(err.ErrorMessage()); 
			return;
		}

	deviceList = _videoProrertyPtr->GetVideoDeviceList();
	_maxVChannelNumber = deviceList.lenght;

	for(int i = 0; i < _maxVChannelNumber; i++)
	{
		int start = 0;
		int position = -1;

		_videoDeviceList[i].Format("%S", deviceList.deviceArray[i].deviceName); 

		position = _videoDeviceList[i].FindOneOf(":\?/*""<>|");
		
		if(position != -1)
			_videoDeviceList[i].SetAt(position, '-');
	}
}
void CVideoManager::PrepareVideoDeviceName()
{
	int start = 0;
	int position = -1;

	_videoDeviceName.Empty(); 

	if(!_videoModulePtr.GetInterfacePtr())
		return;

	BSTR name = _videoModulePtr->GetVideoDeviceName();

	_videoDeviceName.Format("%S", name); 

	position = _videoDeviceName.FindOneOf(":\?/*""<>|");

	if(position != -1)
		_videoDeviceName.SetAt(position, '-');
}
/**********************************************************************/
bool CVideoManager::InitVideoStream(SourceDlg dialog, int videoChannel)
{
	VideoStartData startData;
	bool startResult = false;
	BOOL openStatus = FALSE;

	_isVideoStreamOpen = false;

	if(videoChannel >= _maxVChannelNumber)
		videoChannel = 0;

	if(_videoModulePtr.GetInterfacePtr())
		_videoModulePtr.Release();

	try
	{
		_videoModulePtr = IVideoModulePtr(__uuidof(VideoModule));
	}
	catch(_com_error err)
		{
			AfxMessageBox(err.ErrorMessage()); 
			return false;
		}
      
	startData.dialog = dialog;

	if(videoChannel <= -1)
		startData.videoChannel = 0;
	else
		startData.videoChannel = videoChannel;

	_videoModulePtr->OpenVideoStream(startData);    
	openStatus = _videoModulePtr->IsVideoStreamOpen(); 

	if(!openStatus)
		return false;

	_isVideoStreamOpen  = true;
	_videoChannelNumber = startData.videoChannel;

	_colorHeader = (BITMAPINFO*)_videoModulePtr->GetColorHeader();
	_grayHeader  = (BITMAPINFO*)_videoModulePtr->GetGrayHeader();

	_colorImageSize = _videoModulePtr->GetColorVideoImageSize();
	_grayImageSize  = _videoModulePtr->GetGrayVideoImageSize();

	_colorImageData = _videoModulePtr->GetColorVideoData();
	_grayImageData  = _videoModulePtr->GetGrayVideoData();

	PrepareVideoDeviceName();

	return _isVideoStreamOpen;
}
bool CVideoManager::StopVideoStream()
{
	if(!_videoModulePtr.GetInterfacePtr())
		return false;

	_videoModulePtr->StopVideoStream();
	_videoModulePtr.Release(); 

	return true;
}
/**********************************************************************/
int CVideoManager::NextFrame()
{
	int result = -1;

	if(!_videoModulePtr.GetInterfacePtr())
		return result;

	_videoModulePtr->NextFrame();
	result = _videoModulePtr->GetVideoStatus();

	if(result != 1)
		return result;

	_colorImageSize = _videoModulePtr->GetColorVideoImageSize();
	_grayImageSize  = _videoModulePtr->GetGrayVideoImageSize();

	_colorImageData = _videoModulePtr->GetColorVideoData();
	_grayImageData  = _videoModulePtr->GetGrayVideoData();

	return result;
}
/**********************************************************************/
CString CVideoManager::GetVideoDeviceName(int index)
{
	if(index < 0 || index > _maxVChannelNumber)
		return "";

	return _videoDeviceList[index];
}