// SaveLiveVideoView.cpp : implementation of the CSaveLiveVideoView class
//

#include "stdafx.h"
#include "SaveLiveVideo.h"

#include "SaveLiveVideoDoc.h"
#include "SaveLiveVideoView.h"

#include "MemoryDC.h"
#include "VideoManager.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/********************************************************************************/
IMPLEMENT_DYNCREATE(CSaveLiveVideoView, CView)	// CSaveLiveVideoView
/********************************************************************************/
BEGIN_MESSAGE_MAP(CSaveLiveVideoView, CView)
END_MESSAGE_MAP()
/********************************************************************************/
CSaveLiveVideoView::CSaveLiveVideoView()	// CSaveLiveVideoView construction/destruction
{
	_memoryDC = NULL;

	_gdiplusToken = 0;

	_viewSize.SetSize(0, 0);

	Gdiplus::GdiplusStartupInput gdiplusStartupInput;
	Gdiplus::GdiplusStartup(&_gdiplusToken, &gdiplusStartupInput, NULL);
}

CSaveLiveVideoView::~CSaveLiveVideoView()
{
	GdiplusShutdown(_gdiplusToken);

	CSaveLiveVideoDoc* pDoc = GetDocument();
	pDoc->StopVideoThread();

	SAFE_DELETE(_memoryDC);
}
/********************************************************************************/
void CSaveLiveVideoView::DrawNextFrameDC(CDC* pDC)
{
	CRect clipBoxRect,oldClipBoxRect;

	clipBoxRect.SetRectEmpty();
	oldClipBoxRect.SetRectEmpty(); 

	if(!_memoryDC)
		_memoryDC = new CMemoryDC(pDC);

	_memoryDC->GetClipBox(&oldClipBoxRect);
	GetClientRect(&clipBoxRect);

	if(oldClipBoxRect != clipBoxRect)
	{
		SAFE_DELETE(_memoryDC);
		_memoryDC = new CMemoryDC(pDC, &clipBoxRect);
	}

	_memoryDC->NextFrame(pDC);
}
void CSaveLiveVideoView::OnDraw(CDC* pDC)	// CSaveLiveVideoView drawing
{
	CSaveLiveVideoDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if (!pDoc)
		return;

	CSize imageSize;
	UCHAR *data = NULL;
	BITMAPINFO *header = NULL;
	Bitmap *imageToShow = NULL;

	DrawNextFrameDC(pDC);

	if(!_memoryDC)
		return;

	Graphics graphDevice(_memoryDC->GetSafeHdc());
	graphDevice.Clear(Color(255,255,255));

	imageSize = pDoc->_videoManager->GetColorImageSize();
	header    = pDoc->_videoManager->GetColorHeader();
	data	  = pDoc->_videoManager->GetColorData();

	imageToShow = Bitmap::FromBITMAPINFO(header, data);
	graphDevice.DrawImage(imageToShow, 10, 10, imageSize.cx, imageSize.cy);

	_memoryDC->Display();

	graphDevice.ReleaseHDC(_memoryDC->GetSafeHdc());
}
/********************************************************************************/
#ifdef _DEBUG	// CSaveLiveVideoView diagnostics
void CSaveLiveVideoView::AssertValid() const { CView::AssertValid(); }
void CSaveLiveVideoView::Dump(CDumpContext& dc) const { CView::Dump(dc); }
CSaveLiveVideoDoc* CSaveLiveVideoView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSaveLiveVideoDoc)));
	return (CSaveLiveVideoDoc*)m_pDocument;
}
#endif //_DEBUG
/********************************************************************************/
void CSaveLiveVideoView::OnInitialUpdate()	// CSaveLiveVideoView message handlers
{
	CView::OnInitialUpdate();

	CSaveLiveVideoDoc* pDoc = GetDocument();
	if(!pDoc)
		return;

	pDoc->SetViewWnd(this);
	pDoc->InitVideoManager();

	_viewSize = pDoc->_videoManager->GetColorImageSize();
}
/********************************************************************************/